#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_unified_edge_profile
short_description: Unified Edge server profile template for Cisco Intersight
description:
  - Creates validated server profile templates for Cisco Unified Edge nodes
    used in AI Factory edge inference deployments.
  - Provides hardware-specific presets for supported Unified Edge platforms
    (UCS XE9305, XE130c M8, XE150c M8) with appropriate thermal limits,
    power budgets, and expansion constraints.
  - Attaches user-specified policies for BIOS, boot, storage, network,
    and other configuration via the PolicyBucket mechanism.
  - For generic server profile template management, use
    M(cisco.intersight.intersight_server_profile_template) instead.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
    type: str
    default: default
  name:
    description:
      - The name assigned to the server profile template.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the server profile template.
    type: str
    aliases: [descr]
  hardware_platform:
    description:
      - The Unified Edge hardware platform this template targets.
      - C(xe9305) targets the UCS XE9305 compact edge server.
        Designed for single-GPU or CPU-only inference at the edge with constrained
        thermal envelope and limited PCIe expansion.
      - C(xe130c_m8) targets the UCS XE130c M8 short-depth edge server.
        Supports up to 2 GPUs with moderate thermal headroom for inference workloads
        in ruggedized edge environments.
      - C(xe150c_m8) targets the UCS XE150c M8 edge compute server.
        Full-height edge server supporting up to 4 GPUs with higher power budget
        for demanding edge AI inference and light training workloads.
    type: str
    choices: [xe9305, xe130c_m8, xe150c_m8]
  bios_policy:
    description:
      - Name of BIOS Policy to associate with this template.
      - Consider using a policy created by M(cisco.intersight.intersight_bios_ai_tuning)
        with the C(edge_ai) tuning profile.
    type: str
  boot_order_policy:
    description:
      - Name of Boot Order Policy to associate with this template.
    type: str
  firmware_policy:
    description:
      - Name of Firmware Policy to associate with this template.
    type: str
  storage_policy:
    description:
      - Name of Storage Policy to associate with this template.
      - Consider using a policy created by M(cisco.intersight.intersight_ai_storage_policy)
        with the C(edge_compact) storage profile.
    type: str
  thermal_policy:
    description:
      - Name of Thermal Policy to associate with this template.
      - If not specified, the hardware_platform preset will suggest an appropriate thermal mode.
    type: str
  power_policy:
    description:
      - Name of Power Policy to associate with this template.
    type: str
  lan_connectivity_policy:
    description:
      - Name of LAN Connectivity Policy to associate with this template.
    type: str
  ipmi_over_lan_policy:
    description:
      - Name of IPMI over LAN Policy to associate with this template.
    type: str
  kvm_policy:
    description:
      - Name of Virtual KVM Policy to associate with this template.
    type: str
  network_connectivity_policy:
    description:
      - Name of Network Connectivity Policy to associate with this template.
    type: str
  ntp_policy:
    description:
      - Name of NTP Policy to associate with this template.
    type: str
  snmp_policy:
    description:
      - Name of SNMP Policy to associate with this template.
    type: str
  ssh_policy:
    description:
      - Name of SSH Policy to associate with this template.
    type: str
  syslog_policy:
    description:
      - Name of Syslog Policy to associate with this template.
    type: str
  local_user_policy:
    description:
      - Name of Local User Policy to associate with this template.
    type: str
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Create XE9305 edge inference profile template
  cisco.intersight.intersight_unified_edge_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: Edge-Sites
    name: spt-xe9305-inference
    description: XE9305 compact edge inference server
    hardware_platform: xe9305
    bios_policy: bios-edge-ai
    boot_order_policy: boot-uefi-nvme
    storage_policy: storage-edge-inference
    lan_connectivity_policy: lan-edge-mgmt

- name: Create XE130c M8 edge GPU inference template
  cisco.intersight.intersight_unified_edge_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: spt-xe130c-gpu-inference
    description: XE130c M8 with 2 GPUs for edge inference
    hardware_platform: xe130c_m8
    bios_policy: bios-edge-ai
    storage_policy: storage-model-cache
    firmware_policy: fw-latest

- name: Create XE150c M8 high-performance edge template
  cisco.intersight.intersight_unified_edge_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: spt-xe150c-training
    description: XE150c M8 for edge training and inference
    hardware_platform: xe150c_m8
    bios_policy: bios-gpu-inference-h100
    storage_policy: storage-ai-inference

- name: Delete edge profile template
  cisco.intersight.intersight_unified_edge_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: spt-xe9305-inference
    hardware_platform: xe9305
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "spt-xe9305-inference",
        "ObjectType": "server.ProfileTemplate",
        "TargetPlatform": "UnifiedEdgeServer"
    }
hardware_constraints:
  description: Hardware constraints and recommendations for the selected platform.
  returned: when state is present
  type: dict
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import (
    IntersightModule,
    intersight_argument_spec,
    resolve_policy_bucket,
    sync_policy_bucket,
)


# Hardware platform constraints and metadata
HARDWARE_PLATFORMS = {
    'xe9305': {
        'description': 'UCS XE9305 compact edge server',
        'max_gpus': 0,
        'max_power_watts': 500,
        'thermal_recommendation': 'Acoustic',
        'form_factor': 'compact',
        'pcie_slots': 1,
    },
    'xe130c_m8': {
        'description': 'UCS XE130c M8 short-depth edge server',
        'max_gpus': 2,
        'max_power_watts': 1200,
        'thermal_recommendation': 'Balanced',
        'form_factor': 'short-depth',
        'pcie_slots': 3,
    },
    'xe150c_m8': {
        'description': 'UCS XE150c M8 edge compute server',
        'max_gpus': 4,
        'max_power_watts': 2400,
        'thermal_recommendation': 'HighPower',
        'form_factor': 'full-height',
        'pcie_slots': 6,
    },
}

# Policy mapping for PolicyBucket resolution
POLICY_MAPPING = {
    'bios_policy': {'resource_path': '/bios/Policies', 'object_type': 'bios.Policy'},
    'boot_order_policy': {'resource_path': '/boot/PrecisionPolicies', 'object_type': 'boot.PrecisionPolicy'},
    'firmware_policy': {'resource_path': '/firmware/Policies', 'object_type': 'firmware.Policy'},
    'storage_policy': {'resource_path': '/storage/StoragePolicies', 'object_type': 'storage.StoragePolicy'},
    'thermal_policy': {'resource_path': '/thermal/Policies', 'object_type': 'thermal.Policy'},
    'power_policy': {'resource_path': '/power/Policies', 'object_type': 'power.Policy'},
    'lan_connectivity_policy': {'resource_path': '/vnic/LanConnectivityPolicies', 'object_type': 'vnic.LanConnectivityPolicy'},
    'ipmi_over_lan_policy': {'resource_path': '/ipmioverlan/Policies', 'object_type': 'ipmioverlan.Policy'},
    'kvm_policy': {'resource_path': '/kvm/Policies', 'object_type': 'kvm.Policy'},
    'network_connectivity_policy': {'resource_path': '/networkconfig/Policies', 'object_type': 'networkconfig.Policy'},
    'ntp_policy': {'resource_path': '/ntp/Policies', 'object_type': 'ntp.Policy'},
    'snmp_policy': {'resource_path': '/snmp/Policies', 'object_type': 'snmp.Policy'},
    'ssh_policy': {'resource_path': '/ssh/Policies', 'object_type': 'ssh.Policy'},
    'syslog_policy': {'resource_path': '/syslog/Policies', 'object_type': 'syslog.Policy'},
    'local_user_policy': {'resource_path': '/iam/EndPointUserPolicies', 'object_type': 'iam.EndPointUserPolicy'},
}


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        hardware_platform=dict(type='str', choices=['xe9305', 'xe130c_m8', 'xe150c_m8']),
        bios_policy=dict(type='str'),
        boot_order_policy=dict(type='str'),
        firmware_policy=dict(type='str'),
        storage_policy=dict(type='str'),
        thermal_policy=dict(type='str'),
        power_policy=dict(type='str'),
        lan_connectivity_policy=dict(type='str'),
        ipmi_over_lan_policy=dict(type='str'),
        kvm_policy=dict(type='str'),
        network_connectivity_policy=dict(type='str'),
        ntp_policy=dict(type='str'),
        snmp_policy=dict(type='str'),
        ssh_policy=dict(type='str'),
        syslog_policy=dict(type='str'),
        local_user_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/server/ProfileTemplates'

    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if module.params['state'] == 'present':
        if not module.params.get('hardware_platform'):
            module.fail_json(msg="hardware_platform is required when state is 'present'")

        intersight.set_tags_and_description()

        # Always set target platform to Unified Edge
        intersight.api_body['TargetPlatform'] = 'UnifiedEdgeServer'

        # Expose hardware constraints to the user
        platform = module.params['hardware_platform']
        intersight.result['hardware_constraints'] = HARDWARE_PLATFORMS[platform]

    intersight.configure_policy_or_profile(resource_path=resource_path)
    template_response = intersight.result['api_response']
    template_moid = template_response.get('Moid') if template_response else None

    # Sync PolicyBucket for attached policies
    if template_moid and module.params['state'] == 'present':
        organization_name = module.params['organization']
        desired_bucket = resolve_policy_bucket(intersight, organization_name, POLICY_MAPPING)
        current_bucket = template_response.get('PolicyBucket') or []
        bucket_path = '/server/ProfileTemplates/' + template_moid + '/PolicyBucket'
        sync_policy_bucket(intersight, bucket_path, desired_bucket, current_bucket)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2025 Markus Bergholz
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import json
import typing as t
from urllib.error import HTTPError

from ansible.module_utils.common.text.converters import to_native
from ansible.module_utils.urls import Request

from ansible_collections.community.dns.plugins.module_utils._argspec import ArgumentSpec

if t.TYPE_CHECKING:
    _P = t.ParamSpec("_P")  # pragma: no cover

    FailJson = t.Callable[_P, t.NoReturn]  # pragma: no cover


def create_adguardhome_argument_spec() -> ArgumentSpec:
    return ArgumentSpec(
        {
            "username": {"type": "str", "required": True},
            "password": {"type": "str", "required": True, "no_log": True},
            "host": {"type": "str", "required": True},
            "validate_certs": {"type": "bool", "default": True},
        }
    )


class AdGuardHomeAPIHandler:
    def __init__(self, params: dict[str, t.Any], fail_json: FailJson) -> None:
        host: str = params["host"]  # type: ignore
        self.url = f"{host}/control/rewrite"

        self.validate_certs: bool = params["validate_certs"]  # type: ignore
        self.fail_json = fail_json
        self.r = Request(
            validate_certs=self.validate_certs,
            url_username=params["username"],
            url_password=params["password"],
            force_basic_auth=True,
            headers={"Content-Type": "application/json"},
        )

    def list(self) -> list[dict[str, t.Any]]:
        try:
            response = self.r.open("GET", f"{self.url}/list")

            return json.loads(response.read().decode("utf-8"))

        except HTTPError as e:
            self.fail_json(msg=to_native(e.read()))

    def add_or_delete(
        self,
        domain: str,
        answer: str,
        method: t.Literal["add", "delete"],
        target: dict[str, t.Any],
    ) -> t.Literal[True]:
        """
        the delete api requires the matching answer value.
        but because we make the answer value optional, it's
        taken from previous `find_and_compare` function.
        """
        if method == "add":  # noqa: SIM108
            answer_value = answer
        else:
            answer_value = target["answer"] if answer is None else answer

        data = json.dumps({"domain": domain, "answer": answer_value}).encode("utf-8")
        try:
            self.r.open(
                "POST",
                f"{self.url}/{method}",
                data=data,
            )
            return True

        except HTTPError as e:
            self.fail_json(msg=to_native(e.read()))

    def update(
        self, domain: str, answer: str, target: dict[str, t.Any]
    ) -> t.Literal[True]:
        data = json.dumps(
            {"target": target, "update": {"domain": domain, "answer": answer}}
        ).encode("utf-8")
        try:
            self.r.open(
                "PUT",
                f"{self.url}/update",
                data=data,
            )
            return True

        except HTTPError as e:
            self.fail_json(msg=to_native(e.read()))



# -*- coding: utf-8 -*-

# This file is part of Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from ansible_collections.amazon.aws.plugins.module_utils.botocore import ClientType

# pylint: disable-next=unused-import
from ansible_collections.amazon.aws.plugins.module_utils._route53.common import AnsibleRoute53Error
from ansible_collections.amazon.aws.plugins.module_utils._route53.common import Route53ErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


def manage_tags(
    client: ClientType,
    resource_type: str,
    resource_id: str,
    new_tags: dict,
    purge_tags: bool,
    check_mode: bool,
) -> bool:
    if new_tags is None:
        return False

    old_tags = get_tags(client, resource_type, resource_id)
    tags_to_set, tags_to_delete = compare_aws_tags(old_tags, new_tags, purge_tags=purge_tags)

    change_params = {}
    if tags_to_set:
        change_params["AddTags"] = ansible_dict_to_boto3_tag_list(tags_to_set)
    if tags_to_delete:
        change_params["RemoveTagKeys"] = tags_to_delete

    if not change_params:
        return False

    if check_mode:
        return True

    _change_tags_for_resource(client, resource_type, resource_id, **change_params)
    return True


@Route53ErrorHandler.common_error_handler("change tags for resource")
@AWSRetry.jittered_backoff()
def _change_tags_for_resource(client: ClientType, resource_type: str, resource_id: str, **kwargs) -> dict:
    return client.change_tags_for_resource(
        ResourceType=resource_type,
        ResourceId=resource_id,
        **kwargs,
    )


@Route53ErrorHandler.list_error_handler("list tags for resource", {})
@AWSRetry.jittered_backoff()
def get_tags(client: ClientType, resource_type: str, resource_id: str) -> dict:
    tagset = client.list_tags_for_resource(
        ResourceType=resource_type,
        ResourceId=resource_id,
    )
    tags = boto3_tag_list_to_ansible_dict(tagset["ResourceTagSet"]["Tags"])
    return tags



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Kevin Breit (@kbreit)
deprecated:
  alternative: cisco.meraki.networks_syslog_servers
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Allows for creation and management of Syslog servers within Meraki.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_syslog
notes:
  - Changes to existing syslog servers replaces existing configuration.
  - If you need to add to an existing configuration set state to query to gather the
    existing configuration and then modify or add.
options:
  auth_key:
    description:
      - Authentication key provided by the dashboard. Required if environmental variable
        MERAKI_KEY is not set.
    type: str
  net_id:
    description:
      - ID number of a network.
    type: str
  net_name:
    aliases:
      - name
      - network
    description:
      - Name of a network.
    type: str
  servers:
    description:
      - List of syslog server settings
    elements: dict
    suboptions:
      host:
        description:
          - IP address or hostname of Syslog server.
        type: str
      port:
        default: '514'
        description:
          - Port number Syslog server is listening on.
        type: int
      roles:
        description:
          - List of applicable Syslog server roles.
          - Choices can be one of Wireless Event log, Appliance event log, Switch event log, Air Marshal events, Flows, URLs, IDS alerts, Security events
        elements: str
        type: list
    type: list
  state:
    choices:
      - present
      - query
    default: present
    description:
      - Query or edit syslog servers
      - To delete a syslog server, do not include server in list of servers
    type: str
short_description: Manage syslog server settings in the Meraki cloud.
"""

EXAMPLES = r"""
- name: Query syslog configurations on network named MyNet in the YourOrg organization
  meraki_syslog:
    auth_key: abc12345
    state: query
    org_name: YourOrg
    net_name: MyNet
  delegate_to: localhost
- name: Add single syslog server with Appliance event log role
  meraki_syslog:
    auth_key: abc12345
    state: present
    org_name: YourOrg
    net_name: MyNet
    servers:
      - host: 192.0.1.2
        port: 514
        roles:
          - Appliance event log
  delegate_to: localhost
- name: Add multiple syslog servers
  meraki_syslog:
    auth_key: abc12345
    state: present
    org_name: YourOrg
    net_name: MyNet
    servers:
      - host: 192.0.1.2
        port: 514
        roles:
          - Appliance event log
      - host: 192.0.1.3
        port: 514
        roles:
          - Appliance event log
          - Flows
  delegate_to: localhost
"""

RETURN = r"""
data:
    description: Information about the created or manipulated object.
    returned: info
    type: complex
    contains:
      servers:
        description: List of syslog servers.
        returned: info
        type: complex
        contains:
          host:
            description: Hostname or IP address of syslog server.
            returned: success
            type: str
            sample: 192.0.1.1
          port:
            description: Port number for syslog communication.
            returned: success
            type: str
            sample: 443
          roles:
            description: List of roles assigned to syslog server.
            returned: success
            type: list
            sample: "Wireless event log, URLs"
"""

from copy import deepcopy
from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)


def sort_roles(syslog_servers):
    """Accept a full payload and sort roles"""
    # sorted_servers = list()
    for i, server in enumerate(syslog_servers):
        syslog_servers["servers"][i]["roles"] = sorted(
            syslog_servers["servers"][i]["roles"]
        )
    # return sorted_servers


def validate_role_choices(meraki, servers):
    choices = [
        "Wireless event log",
        "Appliance event log",
        "Switch event log",
        "Air Marshal events",
        "Flows",
        "URLs",
        "IDS alerts",
        "Security events",
    ]

    # Change all choices to lowercase for comparison
    for i in range(len(choices)):
        choices[i] = choices[i].lower()
    for server in range(len(servers)):
        for role in servers[server]["roles"]:
            if role.lower() not in choices:
                meraki.fail_json(
                    msg="Invalid role found in {0}.".format(servers[server]["host"])
                )


def normalize_roles(meraki, servers):
    if len(servers["servers"]) > 0:
        for server in range(len(servers)):
            for role in range(len(servers["servers"][server]["roles"])):
                servers["servers"][server]["roles"][role] = servers["servers"][server][
                    "roles"
                ][role].lower()
    return servers


def main():

    # define the available arguments/parameters that a user can pass to
    # the module

    server_arg_spec = dict(
        host=dict(type="str"),
        port=dict(type="int", default="514"),
        roles=dict(
            type="list",
            elements="str",
        ),
    )

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        net_id=dict(type="str"),
        servers=dict(type="list", elements="dict", options=server_arg_spec),
        state=dict(type="str", choices=["present", "query"], default="present"),
        net_name=dict(type="str", aliases=["name", "network"]),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    meraki = MerakiModule(module, function="syslog")
    module.params["follow_redirects"] = "all"
    payload = None

    syslog_urls = {"syslog": "/networks/{net_id}/syslogServers"}
    meraki.url_catalog["query_update"] = syslog_urls

    if not meraki.params["org_name"] and not meraki.params["org_id"]:
        meraki.fail_json(msg="org_name or org_id parameters are required")
    if meraki.params["state"] != "query":
        if not meraki.params["net_name"] and not meraki.params["net_id"]:
            meraki.fail_json(
                msg="net_name or net_id is required for present or absent states"
            )
    if meraki.params["net_name"] and meraki.params["net_id"]:
        meraki.fail_json(msg="net_name and net_id are mutually exclusive")

    # if the user is working with this module in only check mode we do not
    # want to make any changes to the environment, just return the current
    # state with no modifications

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)

    org_id = meraki.params["org_id"]
    if not org_id:
        org_id = meraki.get_org_id(meraki.params["org_name"])
    net_id = meraki.params["net_id"]
    if net_id is None:
        nets = meraki.get_nets(org_id=org_id)
        net_id = meraki.get_net_id(net_name=meraki.params["net_name"], data=nets)

    if meraki.params["state"] == "query":
        path = meraki.construct_path("query_update", net_id=net_id)
        r = meraki.request(path, method="GET")
        if meraki.status == 200:
            meraki.result["data"] = r
    elif meraki.params["state"] == "present":
        # Validate roles
        validate_role_choices(meraki, meraki.params["servers"])

        # Construct payload
        payload = dict()
        payload["servers"] = meraki.params["servers"]

        # Convert port numbers to string for idempotency checks
        for server in payload["servers"]:
            if server["port"]:
                server["port"] = str(server["port"])
        path = meraki.construct_path("query_update", net_id=net_id)
        r = meraki.request(path, method="GET")
        if meraki.status == 200:
            original = r

        # Roles must be sorted since out of order responses may break idempotency check
        # Need to check to make sure servers are actually defined
        if original is not None:
            if len(original["servers"]) > 0:
                sort_roles(original)
        if payload is not None:
            if len(payload["servers"]) > 0:
                sort_roles(payload)

        # Sanitize roles for comparison
        sanitized_original = normalize_roles(meraki, deepcopy(original))
        sanitized_payload = normalize_roles(meraki, deepcopy(payload))

        if meraki.is_update_required(sanitized_original, sanitized_payload):
            if meraki.module.check_mode is True:
                meraki.generate_diff(original, payload)
                original.update(payload)
                meraki.result["data"] = original
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path("query_update", net_id=net_id)
            r = meraki.request(path, method="PUT", payload=json.dumps(payload))
            if meraki.status == 200:
                meraki.generate_diff(original, r)
                meraki.result["data"] = r
                meraki.result["changed"] = True
        else:
            if meraki.module.check_mode is True:
                meraki.result["data"] = original
                meraki.exit_json(**meraki.result)
            meraki.result["data"] = original

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_image_load

short_description: Load docker image(s) from archives

version_added: 1.3.0

description:
  - Load one or multiple Docker images from a C(.tar) archive, and return information on the loaded image(s).
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  idempotent:
    support: none

options:
  path:
    description:
      - The path to the C(.tar) archive to load Docker image(s) from.
    type: path
    required: true

requirements:
  - "Docker API >= 1.25"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_export
  - module: community.docker.docker_image_push
  - module: community.docker.docker_image_remove
  - module: community.docker.docker_image_tag
"""

EXAMPLES = r"""
---
- name: Load all image(s) from the given tar file
  community.docker.docker_image_load:
    path: /path/to/images.tar
  register: result

- name: Print the loaded image names
  ansible.builtin.debug:
    msg: "Loaded the following images: {{ result.image_names | join(', ') }}"
"""

RETURN = r"""
image_names:
  description: List of image names and IDs loaded from the archive.
  returned: success
  type: list
  elements: str
  sample:
    - 'hello-world:latest'
    - 'sha256:e004c2cc521c95383aebb1fb5893719aa7a8eae2e7a71f316a4410784edb00a9'
images:
  description: Image inspection results for the loaded images.
  returned: success
  type: list
  elements: dict
  sample: []
"""

import errno
import traceback
import typing as t

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    is_image_name_id,
)


class ImageManager(DockerBaseClass):
    def __init__(self, client: AnsibleDockerClient, results: dict[str, t.Any]) -> None:
        super().__init__()

        self.client = client
        self.results = results
        parameters = self.client.module.params
        self.check_mode = self.client.check_mode

        self.path = parameters["path"]

        self.load_images()

    @staticmethod
    def _extract_output_line(line: dict[str, t.Any], output: list[str]) -> None:
        """
        Extract text line from stream output and, if found, adds it to output.
        """
        if "stream" in line or "status" in line:
            # Make sure we have a string (assuming that line['stream'] and
            # line['status'] are either not defined, falsish, or a string)
            text_line = line.get("stream") or line.get("status") or ""
            output.extend(text_line.splitlines())

    def load_images(self) -> None:
        """
        Load images from a .tar archive
        """
        # Load image(s) from file
        load_output: list[str] = []
        try:
            self.log(f"Opening image {self.path}")
            with open(self.path, "rb") as image_tar:
                self.log(f"Loading images from {self.path}")
                res = self.client._post(
                    self.client._url("/images/load"), data=image_tar, stream=True
                )
                for line in self.client._stream_helper(res, decode=True):
                    self.log(line, pretty_print=True)
                    self._extract_output_line(line, load_output)
        except EnvironmentError as exc:
            if exc.errno == errno.ENOENT:
                self.client.fail(f"Error opening archive {self.path} - {exc}")
            self.client.fail(
                f"Error loading archive {self.path} - {exc}",
                stdout="\n".join(load_output),
            )
        except Exception as exc:  # pylint: disable=broad-exception-caught
            self.client.fail(
                f"Error loading archive {self.path} - {exc}",
                stdout="\n".join(load_output),
            )

        # Collect loaded images
        loaded_images = []
        for line in load_output:
            if line.startswith("Loaded image:"):
                loaded_images.append(line[len("Loaded image:") :].strip())
            if line.startswith("Loaded image ID:"):
                loaded_images.append(line[len("Loaded image ID:") :].strip())

        if not loaded_images:
            self.client.fail(
                "Detected no loaded images. Archive potentially corrupt?",
                stdout="\n".join(load_output),
            )

        images = []
        for image_name in loaded_images:
            if is_image_name_id(image_name):
                images.append(self.client.find_image_by_id(image_name))
            elif ":" in image_name:
                image_name, tag = image_name.rsplit(":", 1)
                images.append(self.client.find_image(image_name, tag))
            else:
                self.client.module.warn(
                    f'Image name "{image_name}" is neither ID nor has a tag'
                )

        self.results["image_names"] = loaded_images
        self.results["images"] = images
        self.results["changed"] = True
        self.results["stdout"] = "\n".join(load_output)


def main() -> None:
    client = AnsibleDockerClient(
        argument_spec={
            "path": {"type": "path", "required": True},
        },
        supports_check_mode=False,
    )

    try:
        results: dict[str, t.Any] = {
            "image_names": [],
            "images": [],
        }

        ImageManager(client, results)
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright 2025 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_context_info

short_description: Retrieve information on Docker contexts for the current user

version_added: 4.4.0

description:
  - Return information on Docker contexts.
  - This includes some generic information, as well as a RV(contexts[].config) dictionary that can be used for module defaults for all community.docker modules
    that use the C(community.docker.docker) module defaults group.
extends_documentation_fragment:
  - community.docker._attributes
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state

options:
  only_current:
    description:
      - If set to V(true), RV(contexts) will just contain the current context and none else.
      - If set to V(false) (default), RV(contexts) will list all contexts, unless O(name) is specified.
      - Mutually exclusive to O(name).
    type: bool
    default: false
  name:
    description:
      - A specific Docker CLI context to query.
      - The module will fail if this context does not exist. If you simply want to query whether a context exists,
        do not specify this parameter and use Jinja2 to search the resulting list for a context of the given name instead.
      - Mutually exclusive with O(only_current).
    type: str
  cli_context:
    description:
      - Override for the default context's name.
      - This is preferably used for context selection when O(only_current=true),
        and it is used to compute the return values RV(contexts[].current) and RV(current_context_name).
    type: str

author:
  - "Felix Fontein (@felixfontein)"
"""

EXAMPLES = r"""
---
- name: Get infos on contexts
  community.docker.docker_context_info:
  register: result

- name: Show all contexts
  ansible.builtin.debug:
    msg: "{{ result.contexts }}"

- name: Get current context
  community.docker.docker_context_info:
    only_current: true
  register: docker_current_context

- name: Run community.docker modules with current context
  module_defaults:
    group/community.docker.docker: "{{ docker_current_context.contexts[0].config }}"
  block:
    - name: Task using the current context
      community.docker.docker_container:
        image: ubuntu:latest
        name: ubuntu
        state: started
"""

RETURN = r"""
contexts:
  description:
    - A list of all contexts (O(only_current=false), O(name) not specified),
      only the current context (O(only_current=true)),
      or the requested context (O(name) specified).
  type: list
  elements: dict
  returned: success
  contains:
    current:
      description:
        - Whether this context is the current one.
      type: bool
      returned: success
      sample: true
    name:
      description:
        - The context's name.
      type: bool
      returned: success
      sample: default
    description:
      description:
        - The context's description, if available.
      type: bool
      returned: success
      sample: My context
    meta_path:
      description:
        - The path to the context's meta directory.
        - Not present for RV(contexts[].name=default).
      type: str
      returned: success
      sample: /home/felix/.docker/contexts/meta/0123456789abcdef01234567890abcdef0123456789abcdef0123456789abcde
    tls_path:
      description:
        - The path to the context's TLS config directory.
        - Not present for RV(contexts[].name=default).
      type: str
      returned: success
      sample: /home/user/.docker/contexts/tls/0123456789abcdef01234567890abcdef0123456789abcdef0123456789abcde/
    config:
      description:
        - In case the context is for Docker, contains option values to configure the community.docker modules to use this context.
        - Note that the exact values returned here and their values might change over time if incompatibilities to existing modules are found.
          The goal is that this configuration works fine with all modules in this collection, but we do not have the capabilities to
          test all possible configuration options at the moment.
      type: dict
      returned: success
      sample: {}
      contains:
        docker_host:
          description:
            - The Docker daemon to connect to.
          type: str
          returned: success and context is for Docker
          sample: unix:///var/run/docker.sock
        tls:
          description:
            - Whether the Docker context should use an unvalidated TLS connection.
          type: bool
          returned: success and context is for Docker
          sample: false
        ca_path:
          description:
            - The CA certificate used to validate the Docker daemon's certificate.
          type: bool
          returned: success, context is for Docker, TLS config is present, and CA cert is present
          sample: /path/to/ca-cert.pem
        client_cert:
          description:
            - The client certificate to authenticate with to the Docker daemon.
          type: bool
          returned: success, context is for Docker, TLS config is present, and client cert info is present
          sample: /path/to/client-cert.pem
        client_key:
          description:
            - The client certificate's key to authenticate with to the Docker daemon.
          type: bool
          returned: success, context is for Docker, TLS config is present, and client cert info is present
          sample: /path/to/client-key.pem
        validate_certs:
          description:
            - Whether the Docker context should use a validated TLS connection.
          type: bool
          returned: success, context is for Docker, and TLS config is present
          sample: true

current_context_name:
  description:
    - The name of the current Docker context.
  type: str
  returned: success
  sample: default
"""

import traceback
import typing as t

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.docker.plugins.module_utils._api.context.api import (
    ContextAPI,
)
from ansible_collections.community.docker.plugins.module_utils._api.context.config import (
    get_current_context_name_with_source,
)
from ansible_collections.community.docker.plugins.module_utils._api.context.context import (
    IN_MEMORY,
    Context,
)
from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    ContextException,
    DockerException,
)

if t.TYPE_CHECKING:
    from ansible_collections.community.docker.plugins.module_utils._api.tls import (
        TLSConfig,
    )


def tls_context_to_json(context: TLSConfig | None) -> dict[str, t.Any] | None:
    if context is None:
        return None
    return {
        "client_cert": context.cert[0] if context.cert else None,
        "client_key": context.cert[1] if context.cert else None,
        "ca_cert": context.ca_cert,
        "verify": context.verify,
        # 'ssl_version': context.ssl_version,  -- this isn't used anymore
    }


def context_to_json(context: Context, current: bool) -> dict[str, t.Any]:
    module_config: dict[str, t.Any] = {}
    if "docker" in context.endpoints:
        endpoint = context.endpoints["docker"]
        if isinstance(endpoint.get("Host"), str):
            host_str = to_text(endpoint["Host"])

            # Adjust protocol name so that it works with the Docker CLI tool as well
            proto = None
            idx = host_str.find("://")
            if idx >= 0:
                proto = host_str[:idx]
                host_str = host_str[idx + 3 :]
            if proto in ("http", "https"):
                proto = "tcp"
            if proto == "http+unix":
                proto = "unix"
            if proto:
                host_str = f"{proto}://{host_str}"

            # Create config for the modules
            module_config["docker_host"] = host_str
            if context.tls_cfg.get("docker"):
                tls_cfg = context.tls_cfg["docker"]
                if tls_cfg.ca_cert:
                    module_config["ca_path"] = tls_cfg.ca_cert
                if tls_cfg.cert:
                    module_config["client_cert"] = tls_cfg.cert[0]
                    module_config["client_key"] = tls_cfg.cert[1]
                module_config["validate_certs"] = tls_cfg.verify
                module_config["tls"] = True
            else:
                module_config["tls"] = bool(endpoint.get("SkipTLSVerify"))
    return {
        "current": current,
        "name": context.name,
        "description": context.description,
        "meta_path": None if context.meta_path is IN_MEMORY else context.meta_path,
        "tls_path": None if context.tls_path is IN_MEMORY else context.tls_path,
        "config": module_config,
    }


def main() -> None:
    argument_spec = {
        "only_current": {"type": "bool", "default": False},
        "name": {"type": "str"},
        "cli_context": {"type": "str"},
    }

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("only_current", "name"),
        ],
    )

    only_current: bool = module.params["only_current"]
    name: str | None = module.params["name"]
    cli_context: str | None = module.params["cli_context"]
    try:
        if cli_context:
            current_context_name, current_context_source = (
                cli_context,
                "cli_context module option",
            )
        else:
            current_context_name, current_context_source = (
                get_current_context_name_with_source()
            )
        if name:
            context_or_none = ContextAPI.get_context(name)
            if not context_or_none:
                module.fail_json(msg=f"There is no context of name {name!r}")
            contexts = [context_or_none]
        elif only_current:
            context_or_none = ContextAPI.get_context(current_context_name)
            if not context_or_none:
                module.fail_json(
                    msg=f"There is no context of name {current_context_name!r}, which is configured as the default context ({current_context_source})",
                )
            contexts = [context_or_none]
        else:
            contexts = ContextAPI.contexts()

        json_contexts = sorted(
            [
                context_to_json(context, context.name == current_context_name)
                for context in contexts
            ],
            key=lambda entry: entry["name"],
        )

        module.exit_json(
            changed=False,
            contexts=json_contexts,
            current_context_name=current_context_name,
        )
    except ContextException as e:
        module.fail_json(
            msg=f"Error when handling Docker contexts: {e}",
            exception=traceback.format_exc(),
        )
    except DockerException as e:
        module.fail_json(
            msg=f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_cellular_gateway_uplink_statuses_info
short_description: Information module for organizations _cellular _gateway _uplink
  _statuses
description:
  - Get all organizations _cellular _gateway _uplink _statuses.
  - List the uplink status of every Meraki MG cellular gateway in the organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - >
        NetworkIds query parameter. A list of network IDs. The returned devices will
        be filtered to only include these networks.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. A list of serial numbers. The returned devices will
        be filtered to only include these serials.
    elements: str
    type: list
  iccids:
    description:
      - Iccids query parameter. A list of ICCIDs. The returned devices will be filtered
        to only include these ICCIDs.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for cellularGateway getOrganizationCellularGatewayUplinkStatuses
    description: Complete reference of the getOrganizationCellularGatewayUplinkStatuses
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-uplink-statuses
notes:
  - SDK Method used are
    cellular_gateway.CellularGateway.get_organization_cellular_gateway_uplink_statuses,
  - Paths used are
    get /organizations/{organizationId}/cellularGateway/uplink/statuses,
"""

EXAMPLES = r"""
- name: Get all organizations _cellular _gateway _uplink _statuses
  cisco.meraki.organizations_cellular_gateway_uplink_statuses_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    serials: []
    iccids: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "networkId": "string",
        "serial": "string",
        "model": "string",
        "lastReportedAt": "string",
        "uplinks": [
          {
            "interface": "string",
            "status": "string",
            "ip": "string",
            "provider": "string",
            "publicIp": "string",
            "model": "string",
            "signalStat": {
              "rsrp": "string",
              "rsrq": "string"
            },
            "mcc": "string",
            "mnc": "string",
            "roaming": {
              "status": "string"
            },
            "connectionType": "string",
            "apn": "string",
            "gateway": "string",
            "dns1": "string",
            "dns2": "string",
            "signalType": "string",
            "mtu": 0,
            "iccid": "string",
            "imsi": "string",
            "msisdn": "string"
          }
        ]
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_sensor_commands_info
short_description: Information module for devices _sensor _commands
description:
  - Get all devices _sensor _commands.
  - Get devices _sensor _commands by id.
  - Returns a historical log of all commands.
  - Returns information about the command's execution, including the status.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
      - Serial path parameter.
    type: str
  operations:
    description:
      - >
        Operations query parameter. Optional parameter to filter commands by operation.
        Allowed values are disableDownstreamPower, enableDownstreamPower, cycleDownstreamPower,
        and refreshData.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 10.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  sortOrder:
    description:
      - >
        SortOrder query parameter. Sorted order of entries. Order options are 'ascending'
        and 'descending'. Default is 'descending'.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 30 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 30 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 30 days. The default is 30 days.
    type: float
  commandId:
    description:
      - CommandId path parameter. Command ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sensor getDeviceSensorCommand
    description: Complete reference of the getDeviceSensorCommand API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-sensor-command
  - name: Cisco Meraki documentation for sensor getDeviceSensorCommands
    description: Complete reference of the getDeviceSensorCommands API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-sensor-commands
notes:
  - SDK Method used are
    sensor.Sensor.get_device_sensor_command,
    sensor.Sensor.get_device_sensor_commands,
  - Paths used are
    get /devices/{serial}/sensor/commands,
    get /devices/{serial}/sensor/commands/{commandId},
"""

EXAMPLES = r"""
- name: Get all devices _sensor _commands
  cisco.meraki.devices_sensor_commands_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    operations: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    sortOrder: string
    t0: string
    t1: string
    timespan: 0
    serial: string
    total_pages: -1
    direction: next
  register: result
- name: Get devices _sensor _commands by id
  cisco.meraki.devices_sensor_commands_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serial: string
    commandId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "commandId": "string",
      "createdAt": "string",
      "completedAt": "string",
      "createdBy": {
        "adminId": "string",
        "name": "string",
        "email": "string"
      },
      "operation": "string",
      "status": "string",
      "errors": [
        "string"
      ]
    }
"""



# Copyright (c) 2026, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

# Maps kopia_repository module state values to kopia CLI subcommands.
# Used with cmd_runner_fmt.as_map() for the 'state' arg format.
REPOSITORY_STATE_MAP = {
    "created": "create",
    "connected": "connect",
    "disconnected": "disconnect",
    "synced": "sync-to",
    "throttled": "throttle",
}

# Maps backend provider names to their CLI flag names.
# Each provider emits its name as a positional sub-subcommand, followed by
# --flag value pairs for each non-None param.
# The "server" provider is intentionally absent: `kopia repository connect server`
# uses top-level flags (--url, --server-cert-fingerprint) rather than backend flags,
# so fmt_backend() returns [] for it and those flags are passed separately.
_PROVIDER_BACKEND_MAP = {
    "azure": {
        "container": "--container",
        "storage_account": "--storage-account",
        "storage_key": "--storage-key",
        "sas_token": "--sas-token",
        "storage_domain": "--storage-domain",
        "prefix": "--prefix",
    },
    "b2": {
        "bucket": "--bucket",
        "access_key": "--key-id",
        "secret_access_key": "--key",
        "prefix": "--prefix",
    },
    "filesystem": {
        "path": "--path",
    },
    "gcs": {
        "bucket": "--bucket",
        "credentials_file": "--credentials-file",
        "prefix": "--prefix",
    },
    "gdrive": {
        "folder_id": "--folder-id",
        "credentials_file": "--credentials-file",
    },
    "rclone": {
        "path": "--remote-path",
    },
    "s3": {
        "bucket": "--bucket",
        "access_key": "--access-key",
        "secret_access_key": "--secret-access-key",
        "endpoint": "--endpoint",
        "region": "--region",
        "prefix": "--prefix",
        "session_token": "--session-token",
    },
    "sftp": {
        "path": "--path",
        "host": "--host",
        "username": "--username",
        "port": "--port",
        "keyfile": "--keyfile",
        "known_hosts": "--known-hosts",
    },
    "webdav": {
        "url": "--url",
        "webdav_username": "--webdav-username",
        "webdav_password": "--webdav-password",
    },
}

# Argument spec entries shared by all kopia modules.
# Include this in each module's argument_spec via dict unpacking.
KOPIA_COMMON_ARGUMENT_SPEC = dict(
    password=dict(type="str", no_log=True),
    config=dict(type="path"),
)


def fmt_backend(value):
    """Format the backend dict into positional + flag arguments for kopia CLI.

    For most providers the output is:
        [provider_name, --flag1, value1, --flag2, value2, ...]

    For the "server" provider, returns [] because server connect uses top-level
    flags (--url, --server-cert-fingerprint) passed separately.
    """
    provider = value["provider"]
    if provider == "server":
        return []
    result = [provider]
    for param_name, flag in _PROVIDER_BACKEND_MAP[provider].items():
        param_value = value.get(param_name)
        if param_value is not None:
            result.append(f"{flag}={param_value}")
    return result


def _fmt_throttle(value):
    """Format the throttle dict into --flag value arguments for kopia repository throttle set."""
    if not value:
        return []
    flag_map = {
        "download_bytes_per_second": "--download-bytes-per-second",
        "upload_bytes_per_second": "--upload-bytes-per-second",
        "read_requests_per_second": "--read-requests-per-second",
        "write_requests_per_second": "--write-requests-per-second",
        "list_requests_per_second": "--list-requests-per-second",
        "concurrent_reads": "--concurrent-reads",
        "concurrent_writes": "--concurrent-writes",
    }
    result = []
    for param, flag in flag_map.items():
        v = value.get(param)
        if v is not None:
            result.extend([flag, str(v)])
    return result


def kopia_runner(module: AnsibleModule, extra_formats: dict | None = None, **kwargs) -> CmdRunner:
    """Create a CmdRunner for the kopia CLI.

    Provides arg formats for all params shared across kopia modules.
    Pass extra_formats to add module-specific arg formats on top of the shared ones.
    """
    formats = dict(
        cli_action=cmd_runner_fmt.as_list(),
        status=cmd_runner_fmt.as_fixed("repository", "status"),
        get_throttle=cmd_runner_fmt.as_fixed("repository", "throttle", "get"),
        state=cmd_runner_fmt.as_map(REPOSITORY_STATE_MAP),
        backend=cmd_runner_fmt.as_func(fmt_backend),
        password=cmd_runner_fmt.as_opt_eq_val("--password"),
        fingerprint_tls=cmd_runner_fmt.as_opt_eq_val("--server-cert-fingerprint"),
        url=cmd_runner_fmt.as_opt_eq_val("--url"),
        config=cmd_runner_fmt.as_opt_eq_val("--config-file"),
        throttle_operation=cmd_runner_fmt.as_list(),
        throttle=cmd_runner_fmt.as_func(_fmt_throttle),
    )
    if extra_formats:
        formats.update(extra_formats)
    return CmdRunner(
        module,
        command=["kopia"],
        arg_formats=formats,
        **kwargs,
    )



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_applicationfirewallpolicy_info
version_added: "3.1.0"
short_description: Retrieve Application firewall policy instance facts
description:
    - Get or list the application firewall facts.
options:
    name:
        description:
            - The name of the application firewall policy's name.
        type: str
    resource_group:
        description:
            - The name of the resource group.,
        type: str
    tags:
        description:
            - The application firewall policy's tags key.
            - For filter the resource.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get the application firewall policy by name
  azure_rm_applicationfirewallpolicy_info:
    name: Myfirewallpolicy01
    resource_group: MyResourceGroup

- name: List the application firewall policy by resource group
  azure_rm_applicationfirewallpolicy_info:
    resource_group: MyResourceGroup

- name: List all application firewall policy
  azure_rm_applicationfirewallpolicy_info:
'''

RETURN = '''
firewall_policy:
    description:
        - A list of the application firewall policy facts
    returned: always
    type: complex
    contains:
        id:
            description:
                - The application firewall policy's ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/ApplicationGatewayWebApplicationFirewallPolicies/firewallpolicy"
        name:
            description:
                - Name of application firewall policy.
            returned: always
            type: str
            sample: firewallpolicy
        resource_group:
            description:
                - Name of resource group.
            returned: always
            type: str
            sample: myResourceGroup
        location:
            description:
                - Location of application firewall policy.
            returned: always
            type: str
            sample: eastus
        provisioning_state:
            description:
                - Provisioning state of application firewall policy.
            returned: always
            type: str
            sample: Succeeded
        type:
            description:
                - The type of the application firewall policy.
            returned: always
            type: str
            sample: Microsoft.Network/ApplicationGatewayWebApplicationFirewallPolicies
        tags:
            description:
                - The application firewall policy tags.
            type: dict
            returned: always
            sample: {"key1": "value1"}
        custom_rules:
            description:
                - The custom rules inside the policy.
            type: complex
            returned: when used
            contains:
                action:
                    description:
                        - The name of the resource that is unique within a policy.
                        - This name can be used to access the resource.
                    type: str
                    returned: always
                    sample: Block
                match_conditions:
                    description:
                        - List of match conditions.
                    type: list
                    returned: always
                    sample: [{'match_values': ['10.1.0.4'], 'match_variables': [{'variable_name': 'RemoteAddr'}],
                              'negation_condition': false, 'operator': 'IPMatch', 'transforms': []}]
                name:
                    description:
                        - The name of the resource that is unique within a policy.
                        - This name can be used to access the resource.
                    type: str
                    returned: always
                    sample: testrule01
                priority:
                    description:
                        - Priority of the rule.
                        - Rules with a lower value will be evaluated before rules with a higher value.
                    type: int
                    returned: always
                    sample: 33
                rule_type:
                    description:
                        - The rule type.
                    type: str
                    returned: always
                    sample: MatchRule
                state:
                    description:
                        - Describes if the custom rule is in enabled or disabled state.
                    type: str
                    returned: always
                    sample: Enabled
        managed_rules:
            description:
                - Describes the managedRules structure.
            type: complex
            returned: when used
            contains:
                exclusions:
                    description:
                        - The exceptions that are applied on the policy.
                    type: list
                    returned: always
                    sample: []
                managed_rule_sets:
                    description:
                        - The managed rule sets that are associated with the policy.
                    type: list
                    returned: always
                    sample: [{"rule_group_overrides": [],
                              "rule_set_type": "Microsoft_DefaultRuleSet",
                              "rule_set_version": "2.1"
                             }]
        policy_settings:
            description:
                - The PolicySettings for policy.
            type: complex
            returned: when used
            contains:
                file_upload_enforcement:
                    description:
                        - Whether allow WAF to enforce file upload limits.
                    type: bool
                    returned: always
                    sample: true
                file_upload_limit_in_mb:
                    description:
                        - Maximum file upload size in Mb for WAF.
                    type: int
                    returned: always
                    sample: 100
                js_challenge_cookie_expiration_in_mins:
                    description:
                        - Web Application Firewall JavaScript Challenge Cookie Expiration time in minutes.
                    type: int
                    returned: always
                    sample: 30
                max_request_body_size_in_kb:
                    description:
                        - Maximum request body size in Kb for WAF.
                    type: int
                    returned: always
                    sample: 128
                mode:
                    description:
                        - The mode of the policy.
                    type: str
                    returned: always
                    sample: Detection
                request_body_check:
                    description:
                        - Whether to allow WAF to check request Body.
                    type: bool
                    returned: always
                    sample: false
                request_body_enforcement:
                    description:
                        - Whether allow WAF to enforce request body limits.
                    type: bool
                    returned: always
                    sample: false
                state:
                    description:
                        - The state of the policy.
                    type: str
                    returned: always
                    sample: Enabled
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.core.tools import parse_resource_id
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMApplicationFirewallPolicyInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.tags = None

        super(AzureRMApplicationFirewallPolicyInfo, self).__init__(self.module_arg_spec,
                                                                   supports_check_mode=True,
                                                                   supports_tags=False,
                                                                   facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            if self.resource_group is not None:
                response = self.get()
            else:
                self.fail("Missing resource_group when configed name")
        elif self.resource_group is not None:
            response = self.list_by_rg()
        else:
            response = self.list_all()
        self.results["firewall_policy"] = [item for item in response if self.has_tags(item.get('tags'), self.tags)]
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.network_client.web_application_firewall_policies.get(resource_group_name=self.resource_group, policy_name=self.name)
        except ResourceNotFoundError:
            pass

        if response is not None:
            results.append(self.format_response(response))

        return results

    def list_by_rg(self):
        response = None
        results = []
        try:
            response = self.network_client.web_application_firewall_policies.list(resource_group_name=self.resource_group)
        except Exception as exc:
            self.fail("Error listing web application firewall policy in resource groups {0}: {1}".format(self.resource_group, str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.network_client.web_application_firewall_policies.list_all()
        except Exception as exc:
            self.fail("Error listing all web application firewall policy: {0}".format(str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def format_response(self, item):
        d = item.as_dict()
        id_dict = parse_resource_id(item.id)
        d['resource_group'] = id_dict.get('resource_group')

        return d


def main():
    AzureRMApplicationFirewallPolicyInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022 Ansible Project
# Copyright (c) 2022 Alina Buzachis (@alinabuzachis)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: rds_cluster_info
version_added: 5.0.0
short_description: Obtain information about one or more RDS clusters
description:
  - Obtain information about one or more RDS clusters.
  - This module was originally added to C(community.aws) in release 3.2.0.
options:
    db_cluster_identifier:
        description:
          - The user-supplied DB cluster identifier.
          - If this parameter is specified, information from only the specific DB cluster is returned.
        aliases:
          - cluster_id
          - id
          - cluster_name
        type: str
    filters:
        description:
            - A filter that specifies one or more DB clusters to describe.
              See U(https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_DescribeDBClusters.html).
        type: dict
author:
  - Alina Buzachis (@alinabuzachis)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get info of all existing DB clusters
  amazon.aws.rds_cluster_info:
  register: _result_cluster_info

- name: Get info on a specific DB cluster
  amazon.aws.rds_cluster_info:
    cluster_id: "{{ cluster_id }}"
  register: _result_cluster_info

- name: Get info all DB clusters with specific engine
  amazon.aws.rds_cluster_info:
    engine: "aurora"
  register: _result_cluster_info
"""

RETURN = r"""
clusters:
  description: List of RDS clusters.
  returned: always
  type: list
  contains:
    activity_stream_status:
        description: The status of the database activity stream.
        type: str
        sample: "stopped"
    allocated_storage:
        description:
        - The allocated storage size in gigabytes. Since aurora storage size is not fixed this is
          always 1 for aurora database engines.
        type: int
        sample: 1
    associated_roles:
        description:
        - A list of dictionaries of the AWS Identity and Access Management (IAM) roles that are associated
          with the DB cluster. Each dictionary contains the role_arn and the status of the role.
        type: list
        sample: []
    auto_minor_version_upgrade:
        description: Specifies whether minor engine upgrades are applied automatically to the DB cluster during the maintenance window.
        returned: always
        type: bool
        sample: true
    availability_zones:
        description: The list of availability zones that instances in the DB cluster can be created in.
        type: list
        sample:
        - us-east-1c
        - us-east-1a
        - us-east-1e
    backup_retention_period:
        description: The number of days for which automatic DB snapshots are retained.
        type: int
        sample: 1
    cluster_create_time:
        description: The time in UTC when the DB cluster was created.
        type: str
        sample: "2018-06-29T14:08:58.491000+00:00"
    copy_tags_to_snapshot:
        description:
        - Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.
        type: bool
        sample: false
    cross_account_clone:
        description:
        - Specifies whether the DB cluster is a clone of a DB cluster owned by a different Amazon Web Services account.
        type: bool
        sample: false
    db_cluster_arn:
        description: The Amazon Resource Name (ARN) for the DB cluster.
        type: str
        sample: "arn:aws:rds:us-east-1:123456789012:cluster:rds-cluster-demo"
    db_cluster_identifier:
        description: The lowercase user-supplied DB cluster identifier.
        type: str
        sample: "rds-cluster-demo"
    db_cluster_members:
        description:
        - A list of dictionaries containing information about the instances in the cluster.
          Each dictionary contains the I(db_instance_identifier), I(is_cluster_writer) (bool),
          I(db_cluster_parameter_group_status), and I(promotion_tier) (int).
        type: list
        sample: []
    db_cluster_parameter_group:
        description: The parameter group associated with the DB cluster.
        type: str
        sample: "default.aurora5.6"
    db_cluster_resource_id:
        description: The AWS Region-unique, immutable identifier for the DB cluster.
        type: str
        sample: "cluster-D2MEQDN3BQNXDF74K6DQJTHASU"
    db_subnet_group:
        description: The name of the subnet group associated with the DB Cluster.
        type: str
        sample: "default"
    deletion_protection:
        description:
        - Indicates if the DB cluster has deletion protection enabled.
          The database can't be deleted when deletion protection is enabled.
        type: bool
        sample: false
    domain_memberships:
        description:
        - The Active Directory Domain membership records associated with the DB cluster.
        type: list
        sample: []
    earliest_restorable_time:
        description: The earliest time to which a database can be restored with point-in-time restore.
        type: str
        sample: "2018-06-29T14:09:34.797000+00:00"
    endpoint:
        description: The connection endpoint for the primary instance of the DB cluster.
        type: str
        sample: "rds-cluster-demo.cluster-cvlrtwiennww.us-east-1.rds.amazonaws.com"
    engine:
        description: The database engine of the DB cluster.
        type: str
        sample: "aurora"
    engine_mode:
        description: The DB engine mode of the DB cluster.
        type: str
        sample: "provisioned"
    engine_version:
        description: The database engine version.
        type: str
        sample: "5.6.10a"
    hosted_zone_id:
        description: The ID that Amazon Route 53 assigns when you create a hosted zone.
        type: str
        sample: "Z2R2ITUGPM61AM"
    http_endpoint_enabled:
        description:
        - A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled.
        type: bool
        sample: false
    iam_database_authentication_enabled:
        description: Whether IAM accounts may be mapped to database accounts.
        type: bool
        sample: false
    latest_restorable_time:
        description: The latest time to which a database can be restored with point-in-time restore.
        type: str
        sample: "2018-06-29T14:09:34.797000+00:00"
    master_username:
        description: The master username for the DB cluster.
        type: str
        sample: "username"
    multi_az:
        description: Whether the DB cluster has instances in multiple availability zones.
        type: bool
        sample: false
    port:
        description: The port that the database engine is listening on.
        type: int
        sample: 3306
    preferred_backup_window:
        description: The UTC weekly time range during which system maintenance can occur.
        type: str
        sample: "10:18-10:48"
    preferred_maintenance_window:
        description: The UTC weekly time range during which system maintenance can occur.
        type: str
        sample: "tue:03:23-tue:03:53"
    read_replica_identifiers:
        description: A list of read replica ID strings associated with the DB cluster.
        type: list
        sample: []
    reader_endpoint:
        description: The reader endpoint for the DB cluster.
        type: str
        sample: "rds-cluster-demo.cluster-ro-cvlrtwiennww.us-east-1.rds.amazonaws.com"
    status:
        description: The status of the DB cluster.
        type: str
        sample: "available"
    storage_encrypted:
        description: Whether the DB cluster is storage encrypted.
        type: bool
        sample: false
    tag_list:
        description: A list of tags consisting of key-value pairs.
        type: list
        elements: dict
        sample: [
            {
                "key": "Created_By",
                "value": "Ansible_rds_cluster_integration_test"
            }
        ]
    tags:
        description: A dictionary of key value pairs.
        type: dict
        sample: {
            "Name": "rds-cluster-demo"
        }
    vpc_security_groups:
        description: A list of the DB cluster's security groups and their status.
        type: complex
        contains:
            status:
                description: Status of the security group.
                type: str
                sample: "active"
            vpc_security_group_id:
                description: Security group of the cluster.
                type: str
                sample: "sg-12345678"
"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


@AWSRetry.jittered_backoff(retries=10)
def _describe_db_clusters(client, **params):
    try:
        paginator = client.get_paginator("describe_db_clusters")
        return paginator.paginate(**params).build_full_result()["DBClusters"]
    except is_boto3_error_code("DBClusterNotFoundFault"):
        return []


def cluster_info(client, module):
    cluster_id = module.params.get("db_cluster_identifier")
    filters = module.params.get("filters")

    params = dict()
    if cluster_id:
        params["DBClusterIdentifier"] = cluster_id
    if filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)

    try:
        result = _describe_db_clusters(client, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, "Couldn't get RDS cluster information.")

    for cluster in result:
        cluster["Tags"] = get_tags(client, module, cluster["DBClusterArn"])

    return dict(changed=False, clusters=[camel_dict_to_snake_dict(cluster, ignore_list=["Tags"]) for cluster in result])


def main():
    argument_spec = dict(
        db_cluster_identifier=dict(aliases=["cluster_id", "id", "cluster_name"]),
        filters=dict(type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        client = module.client("rds", retry_decorator=AWSRetry.jittered_backoff(retries=10))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    module.exit_json(**cluster_info(client, module))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_adaptive_policy_groups
short_description: Resource module for organizations _adaptive _policy _groups
description:
  - Manage operations create, update and delete of the resource organizations _adaptive
    _policy _groups.
  - Creates a new adaptive policy group.
  - Deletes the specified adaptive policy group and any associated policies and references.
  - Updates an adaptive policy group. If updating "Infrastructure", only the SGT is
    allowed. Cannot update "Unknown".
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  description:
    description: Description of the group (default "").
    type: str
  id:
    description: Id path parameter.
    type: str
  name:
    description: Name of the group.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  policyObjects:
    description: The policy objects that belong to this group; traffic from addresses
      specified by these policy objects will be tagged with this group's SGT value
      if no other tagging scheme is being used (each requires one unique attribute)
      (default ).
    elements: dict
    suboptions:
      id:
        description: The ID of the policy object.
        type: str
      name:
        description: The name of the policy object.
        type: str
    type: list
  sgt:
    description: SGT value of the group.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationAdaptivePolicyGroup
    description: Complete reference of the createOrganizationAdaptivePolicyGroup API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-adaptive-policy-group
  - name: Cisco Meraki documentation for organizations deleteOrganizationAdaptivePolicyGroup
    description: Complete reference of the deleteOrganizationAdaptivePolicyGroup API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-adaptive-policy-group
  - name: Cisco Meraki documentation for organizations updateOrganizationAdaptivePolicyGroup
    description: Complete reference of the updateOrganizationAdaptivePolicyGroup API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-group
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_adaptive_policy_group,
    organizations.Organizations.delete_organization_adaptive_policy_group,
    organizations.Organizations.update_organization_adaptive_policy_group,
  - Paths used are
    post /organizations/{organizationId}/adaptivePolicy/groups,
    delete
    /organizations/{organizationId}/adaptivePolicy/groups/{id},
    put /organizations/{organizationId}/adaptivePolicy/groups/{id},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_adaptive_policy_groups:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    description: Group of XYZ Corp Employees
    name: Employee Group
    organizationId: string
    policyObjects:
      - id: '2345'
        name: Example Policy Object
    sgt: 1000
- name: Update by id
  cisco.meraki.organizations_adaptive_policy_groups:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    description: Group of XYZ Corp Employees
    id: string
    name: Employee Group
    organizationId: string
    policyObjects:
      - id: '2345'
        name: Example Policy Object
    sgt: 1000
- name: Delete by id
  cisco.meraki.organizations_adaptive_policy_groups:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    id: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "groupId": "string",
      "name": "string",
      "sgt": 0,
      "description": "string",
      "policyObjects": [
        {
          "id": "string",
          "name": "string"
        }
      ],
      "isDefaultGroup": true,
      "requiredIpMappings": [
        "string"
      ],
      "createdAt": "string",
      "updatedAt": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_flow_control_policy
short_description: Flow Control Policy configuration for Cisco Intersight
description:
  - Manages Flow Control Policy configuration on Cisco Intersight.
  - A policy to configure Priority Flow Control (PFC) and link-level flow control settings for Cisco Intersight managed fabric interconnects.
  - Priority Flow Control (PFC) enables no-drop behavior for specific traffic classes, while link-level flow control manages traffic flow at the link layer.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/FlowControlPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Flow Control Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Flow Control Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  priority_flow_control_mode:
    description:
      - Configure the Priority Flow Control (PFC) for each port to enable the no-drop behavior for the CoS defined by the System QoS Policy.
      - PFC works with Ethernet QoS policy to enable no-drop behavior for specific CoS values.
      - If Auto or On is selected for PFC, the Receive and Send link level flow control will be Off.
      - C(auto) - PFC is automatically configured based on the QoS policies (default).
      - C(on) - PFC is always enabled for the configured CoS values.
      - C(off) - PFC is disabled. Link-level flow control settings will be used instead.
    type: str
    choices: ['auto', 'on', 'off']
    default: auto
  receive_direction:
    description:
      - Link-level Flow Control configured in the receive direction.
      - This parameter is only applicable when C(priority_flow_control_mode) is set to C(off).
      - When C(priority_flow_control_mode) is C(auto) or C(on), this will be automatically set to C(disabled).
      - C(enabled) - Enable receive direction flow control.
      - C(disabled) - Disable receive direction flow control.
    type: str
    choices: [enabled, disabled]
    default: enabled
  send_direction:
    description:
      - Link-level Flow Control configured in the send direction.
      - This parameter is only applicable when C(priority_flow_control_mode) is set to C(off).
      - When C(priority_flow_control_mode) is C(auto) or C(on), this will be automatically set to C(disabled).
      - C(enabled) - Enable send direction flow control.
      - C(disabled) - Disable send direction flow control.
    type: str
    choices: [enabled, disabled]
    default: enabled
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Flow Control Policy with auto PFC mode
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "FlowControl-Auto-Policy"
    description: "Flow control policy with auto PFC"
    priority_flow_control_mode: auto
    tags:
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create a Flow Control Policy with PFC enabled
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FlowControl-PFC-On-Policy"
    description: "Flow control policy with PFC always on"
    priority_flow_control_mode: on
    state: present

- name: Create a Flow Control Policy with link-level flow control
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FlowControl-LinkLevel-Policy"
    description: "Flow control policy with link-level settings"
    priority_flow_control_mode: off
    receive_direction: enabled
    send_direction: enabled
    state: present

- name: Create a Flow Control Policy with disabled flow control
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FlowControl-Disabled-Policy"
    priority_flow_control_mode: off
    receive_direction: disabled
    send_direction: disabled
    state: present

- name: Update a Flow Control Policy
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FlowControl-Auto-Policy"
    description: "Updated flow control policy"
    priority_flow_control_mode: on
    state: present

- name: Delete a Flow Control Policy
  cisco.intersight.intersight_flow_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "FlowControl-Auto-Policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "FlowControl-Auto-Policy",
        "ObjectType": "fabric.FlowControlPolicy",
        "PriorityFlowControlMode": "auto",
        "ReceiveDirection": "Enabled",
        "SendDirection": "Enabled",
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        priority_flow_control_mode=dict(type='str', choices=['auto', 'on', 'off'], default='auto'),
        receive_direction=dict(type='str', choices=['enabled', 'disabled'], default='enabled'),
        send_direction=dict(type='str', choices=['enabled', 'disabled'], default='enabled')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/fabric/FlowControlPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Convert lowercase choices to API format
        priority_mode = intersight.module.params['priority_flow_control_mode']
        intersight.api_body['PriorityFlowControlMode'] = priority_mode

        # Set receive and send direction based on priority flow control mode
        # When PFC is auto or on, receive/send must be disabled
        # When PFC is off, use the configured receive/send direction values
        if priority_mode in ['auto', 'on']:
            intersight.api_body['ReceiveDirection'] = 'Disabled'
            intersight.api_body['SendDirection'] = 'Disabled'
        else:
            receive_dir = intersight.module.params['receive_direction']
            intersight.api_body['ReceiveDirection'] = receive_dir.capitalize()
            send_dir = intersight.module.params['send_direction']
            intersight.api_body['SendDirection'] = send_dir.capitalize()

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_aaa_user
short_description: Manage AAA users (aaa:User)
description:
- Manage AAA users on Cisco ACI fabrics.
requirements:
- python-dateutil
options:
  aaa_password:
    description:
    - The password of the locally-authenticated user.
    - Providing this option will always result in a change because it is a secure property that cannot be retrieved from APIC.
    type: str
  aaa_password_lifetime:
    description:
    - The lifetime of the locally-authenticated user password.
    type: int
  aaa_password_update_required:
    description:
    - Whether this account needs password update.
    type: bool
  aaa_user:
    description:
    - The name of the locally-authenticated user user to add.
    type: str
    aliases: [ name ]
  clear_password_history:
    description:
    - Whether to clear the password history of a locally-authenticated user.
    type: bool
  description:
    description:
    - Description for the AAA user.
    type: str
    aliases: [ descr ]
  email:
    description:
    - The email address of the locally-authenticated user.
    type: str
  enabled:
    description:
    - The status of the locally-authenticated user account.
    type: bool
  expiration:
    description:
    - The expiration date of the locally-authenticated user account.
    type: str
  expires:
    description:
    - Whether to enable an expiration date for the locally-authenticated user account.
    type: bool
  first_name:
    description:
    - The first name of the locally-authenticated user.
    type: str
  last_name:
    description:
    - The last name of the locally-authenticated user.
    type: str
  phone:
    description:
    - The phone number of the locally-authenticated user.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- This module is not idempotent when C(aaa_password) is being used
  (even if that password was already set identically). This
  appears to be an inconsistency wrt. the idempotent nature
  of the APIC REST API. The vendor has been informed.
  More information in :ref:`the ACI documentation <aci_guide_known_issues>`.
seealso:
- module: cisco.aci.aci_aaa_user_certificate
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(aaa:User).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
"""

EXAMPLES = r"""
- name: Add a user
  cisco.aci.aci_aaa_user:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: dag
    aaa_password: AnotherSecretPassword
    expiration: never
    expires: false
    email: dag@wieers.com
    phone: 1-234-555-678
    first_name: Dag
    last_name: Wieers
    state: present
  delegate_to: localhost

- name: Remove a user
  cisco.aci.aci_aaa_user:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: dag
    state: absent
  delegate_to: localhost

- name: Query a user
  cisco.aci.aci_aaa_user:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: dag
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all users
  cisco.aci.aci_aaa_user:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: '?rsp-prop-include=config-only'
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

try:
    from dateutil.tz import tzutc
    import dateutil.parser

    HAS_DATEUTIL = True
except ImportError:
    HAS_DATEUTIL = False

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        aaa_password=dict(type="str", no_log=True),
        aaa_password_lifetime=dict(type="int", no_log=False),
        aaa_password_update_required=dict(type="bool", no_log=False),
        aaa_user=dict(type="str", aliases=["name"]),  # Not required for querying all objects
        clear_password_history=dict(type="bool", no_log=False),
        description=dict(type="str", aliases=["descr"]),
        email=dict(type="str"),
        enabled=dict(type="bool"),
        expiration=dict(type="str"),
        expires=dict(type="bool"),
        first_name=dict(type="str"),
        last_name=dict(type="str"),
        phone=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["aaa_user"]],
            ["state", "present", ["aaa_user"]],
            ["expires", True, ["expiration"]],
        ],
    )

    aci = ACIModule(module)

    if not HAS_DATEUTIL:
        module.fail_json(msg="dateutil required for this module")

    aaa_password = module.params.get("aaa_password")
    aaa_password_lifetime = module.params.get("aaa_password_lifetime")
    aaa_password_update_required = aci.boolean(module.params.get("aaa_password_update_required"))
    aaa_user = module.params.get("aaa_user")
    clear_password_history = aci.boolean(module.params.get("clear_password_history"))
    description = module.params.get("description")
    email = module.params.get("email")
    enabled = aci.boolean(module.params.get("enabled"), "active", "inactive")
    expires = aci.boolean(module.params.get("expires"))
    first_name = module.params.get("first_name")
    last_name = module.params.get("last_name")
    phone = module.params.get("phone")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    expiration = module.params.get("expiration")
    if expiration is not None and expiration != "never":
        try:
            expiration = aci.iso8601_format(dateutil.parser.parse(expiration).replace(tzinfo=tzutc()))
        except Exception as e:
            module.fail_json(msg="Failed to parse date format '%s', %s" % (module.params.get("expiration"), e))

    aci.construct_url(
        root_class=dict(
            aci_class="aaaUser",
            aci_rn="userext/user-{0}".format(aaa_user),
            module_object=aaa_user,
            target_filter={"name": aaa_user},
        ),
    )
    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="aaaUser",
            class_config=dict(
                accountStatus=enabled,
                clearPwdHistory=clear_password_history,
                descr=description,
                email=email,
                expiration=expiration,
                expires=expires,
                firstName=first_name,
                lastName=last_name,
                name=aaa_user,
                phone=phone,
                pwd=aaa_password,
                pwdLifeTime=aaa_password_lifetime,
                pwdUpdateRequired=aaa_password_update_required,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class="aaaUser")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Fran Fitzpatrick (francis.x.fitzpatrick@gmail.com)
# Borrowed heavily from other work by Abhijeet Kasurde (akasurde@redhat.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_dnszone
author: Fran Fitzpatrick (@fxfitz)
short_description: Manage FreeIPA DNS Zones
description:
  - Add and delete an IPA DNS Zones using IPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  zone_name:
    description:
      - The DNS zone name to which needs to be managed.
    required: true
    type: str
  state:
    description: State to ensure.
    default: present
    choices: ["absent", "present"]
    type: str
  dynamicupdate:
    description: Apply dynamic update to zone.
    default: false
    type: bool
  allowsyncptr:
    description: Allow synchronization of forward and reverse records in the zone.
    default: false
    type: bool
    version_added: 4.3.0
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure dns zone is present
  community.general.ipa_dnszone:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: present
    zone_name: example.com

- name: Ensure dns zone is present and is dynamic update
  community.general.ipa_dnszone:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: present
    zone_name: example.com
    dynamicupdate: true

- name: Ensure that dns zone is removed
  community.general.ipa_dnszone:
    zone_name: example.com
    ipa_host: localhost
    ipa_user: admin
    ipa_pass: topsecret
    state: absent

- name: Ensure dns zone is present and is allowing sync
  community.general.ipa_dnszone:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: present
    zone_name: example.com
    allowsyncptr: true
"""

RETURN = r"""
zone:
  description: DNS zone as returned by IPA API.
  returned: always
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class DNSZoneIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def dnszone_find(self, zone_name, details=None):
        items = {
            "all": "true",
            "idnsname": zone_name,
        }
        if details is not None:
            items.update(details)

        return self._post_json(method="dnszone_find", name=zone_name, item=items)

    def dnszone_add(self, zone_name=None, details=None):
        items = {}
        if details is not None:
            items.update(details)

        return self._post_json(method="dnszone_add", name=zone_name, item=items)

    def dnszone_mod(self, zone_name=None, details=None):
        items = {}
        if details is not None:
            items.update(details)

        return self._post_json(method="dnszone_mod", name=zone_name, item=items)

    def dnszone_del(self, zone_name=None, record_name=None, details=None):
        return self._post_json(method="dnszone_del", name=zone_name, item={})


def ensure(module, client):
    zone_name = module.params["zone_name"]
    state = module.params["state"]
    dynamicupdate = module.params["dynamicupdate"]
    allowsyncptr = module.params["allowsyncptr"]

    changed = False

    # does zone exist
    ipa_dnszone = client.dnszone_find(zone_name)

    if state == "present":
        if not ipa_dnszone:
            changed = True
            if not module.check_mode:
                client.dnszone_add(
                    zone_name=zone_name, details={"idnsallowdynupdate": dynamicupdate, "idnsallowsyncptr": allowsyncptr}
                )
        elif (
            ipa_dnszone["idnsallowdynupdate"][0] != str(dynamicupdate).upper()
            or ipa_dnszone.get("idnsallowsyncptr")
            and ipa_dnszone["idnsallowsyncptr"][0] != str(allowsyncptr).upper()
        ):
            changed = True
            if not module.check_mode:
                client.dnszone_mod(
                    zone_name=zone_name, details={"idnsallowdynupdate": dynamicupdate, "idnsallowsyncptr": allowsyncptr}
                )
        else:
            changed = False

    # state is absent
    else:
        # check for generic zone existence
        if ipa_dnszone:
            changed = True
            if not module.check_mode:
                client.dnszone_del(zone_name=zone_name)

    return changed, client.dnszone_find(zone_name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        zone_name=dict(type="str", required=True),
        state=dict(type="str", default="present", choices=["present", "absent"]),
        dynamicupdate=dict(type="bool", default=False),
        allowsyncptr=dict(type="bool", default=False),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = DNSZoneIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, zone = ensure(module, client)
        module.exit_json(changed=changed, zone=zone)
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_dedicated_host_info
version_added: 9.3.0
short_description: Gather information about EC2 Dedicated Hosts in AWS
description:
  - Gather information about EC2 Dedicated Hosts in AWS.
  - The module can also gather information about dedicated Mac hosts.
author:
  - "Aubin Bikouo (@abikouo)"
options:
  host_ids:
    description:
      - The IDs of the Dedicated Hosts.
    type: list
    elements: str
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeHosts.html) for possible filters.
      - See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeMacHosts.html) for possible filters for
        EC2 Mac dedicated hosts.
    type: dict
  mac_only:
    description:
      - When set to V(true) retrieve EC2 Mac Dedicated Hosts only.
    type: bool
    default: False
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all dedicated hosts
- name: Gather information about all dedicated hosts
  amazon.aws.ec2_dedicated_host_info:

# Gather information about all EC2 Mac dedicated hosts only
- name: Gather information about all EC2 Mac dedicated hosts
  amazon.aws.ec2_dedicated_host_info:
    mac_only: true

# Gather information about dedicated hosts with state=released
- name: Gather information about released dedicated hosts
  amazon.aws.ec2_dedicated_host_info:
    filters:
      state: released
"""

RETURN = r"""
hosts:
  description: List of dedicated hosts.
  returned: When O(mac_only=false)
  type: complex
  contains:
    allocation_time:
      description: The time that the Dedicated Host was allocated.
      returned: always
      type: str
      sample: "2025-02-12T12:09:22+00:00"
    allows_multiple_instance_types:
      description: Indicates whether the Dedicated Host supports multiple instance types of the same instance family.
      returned: always
      type: str
      sample: "off"
    auto_placement:
      description: Whether auto-placement is on or off.
      returned: always
      type: str
      sample: "off"
    availability_zone:
      description: The Availability Zone of the Dedicated Host.
      returned: always
      type: str
      sample: "us-east-1b"
    availability_zone_id:
      description: The ID of the Availability Zone in which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "use1-az6"
    available_capacity:
      description: Information about the instances running on the Dedicated Host.
      returned: always
      type: dict
      contains:
        available_v_cpus:
          description: The number of vCPUs available for launching instances onto the Dedicated Host.
          returned: always
          type: int
          sample: 8
        available_instance_capacity:
          description: The number of instances that can be launched onto the Dedicated Host.
          returned: always
          type: complex
          contains:
            available_capacity:
              description: The number of instances that can be launched onto the Dedicated Host.
              returned: always
              type: int
              sample: 1
            instance_type:
              description: The instance type supported by the Dedicated Host.
              returned: always
              type: str
              sample: "mac2.metal"
            total_capacity:
              description: The total number of instances that can be launched onto the Dedicated Host if there are no instances running on it.
              returned: always
              type: dict
              sample: 1
    host_id:
      description: The ID of the Dedicated Host.
      returned: always
      type: str
      sample: "h-03f51341e6e39f848"
    client_token:
      description: Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
      returned: always
      type: str
      sample: "token-0123456789a"
    host_properties:
      description: The hardware specifications of the Dedicated Host.
      returned: always
      type: dict
      contains:
        cores:
          description: The number of cores on the Dedicated Host.
          returned: always
          type: int
          sample: 8
        instance_type:
          description: The instance type supported by the Dedicated Host.
          returned: always
          type: str
          sample: "mac2.metal"
        instance_family:
          description: The instance family supported by the Dedicated Host.
          returned: if defined
          type: str
          sample: "mac2"
        sockets:
          description: The number of sockets on the Dedicated Host.
          returned: always
          type: int
          sample: 1
        total_v_cpus:
          description: The total number of vCPUs on the Dedicated Host.
          returned: always
          type: int
          sample: 8
    host_reservation_id:
      description: The reservation ID of the Dedicated Host.
      returned: always
      type: str
    instances:
      description: The IDs and instance type that are currently running on the Dedicated Host.
      returned: always
      type: complex
      contains:
        instance_id:
          description: The ID of instance that is running on the Dedicated Host.
          returned: always
          type: str
          sample: "i-0123456789abcd"
        instance_type:
          description: The instance type of the running instance.
          returned: always
          type: str
          sample: "ec2.micro"
        owner_id:
          description: The ID of the Amazon Web Services account that owns the instance.
          returned: always
          type: str
          sample: "0123456789"
    state:
      description: The state of the Dedicated Host.
      returned: always
      type: str
      sample: "available"
    release_time:
      description: The time that the Dedicated Host was released.
      returned: always
      type: str
      sample: "2025-02-12T12:09:22+00:00"
    host_recovery:
      description: Indicates whether host recovery is enabled or disabled for the Dedicated Host.
      returned: always
      type: str
      sample: "off"
    owner_id:
      description: The ID of the Amazon Web Services account that owns the Dedicated Host.
      returned: always
      type: str
      sample: "0123456789"
    member_of_service_linked_resource_group:
      description: Indicates whether the Dedicated Host is in a host resource group.
      returned: always
      type: bool
      sample: false
    outpost_arn:
      description: The Amazon Resource Name (ARN) of the Amazon Web Services Outpost on which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "arn:aws:outposts:us-east-1:0123012301230123:outpost/op-0123456789abcdef0"
    host_maintenance:
      description: Indicates whether host maintenance is enabled or disabled for the Dedicated Host.
      returned: always
      type: str
      sample: "off"
    asset_id:
      description: The ID of the Outpost hardware asset on which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "abcdefgh"
    tags:
      description: Dictionary of tags added to the dedicated host.
      returned: always
      type: dict
      sample: {}
mac_hosts:
  description: List of EC2 Mac dedicated hosts.
  returned: When O(mac_only=true)
  type: complex
  contains:
    host_id:
      description: The EC2 Mac Dedicated Host ID.
      returned: always
      type: str
      sample: "h-04b38258aba8c1875"
    mac_os_latest_supported_versions:
      description: The latest macOS versions that the EC2 Mac Dedicated Host can launch without being upgraded.
      returned: always
      type: list
      elements: str
      sample: ["15.3", "14.7.3", "13.7.3"]
"""

from typing import Any
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_dedicated_hosts
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_mac_dedicated_hosts
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def format_results(hosts: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    results = []
    for h in hosts:
        h["tags"] = boto3_tag_list_to_ansible_dict(h.pop("Tags", {}))
        results.append(camel_dict_to_snake_dict(h, ignore_list=["tags"]))
    return results


def build_args(module: AnsibleAWSModule) -> Dict[str, Any]:
    filters = module.params.get("filters")
    host_ids = module.params.get("host_ids")
    params = {}
    if host_ids:
        params["HostIds"] = host_ids
    if filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)
    return params


def list_dedicated_hosts(client: Any, module: AnsibleAWSModule) -> Dict[str, Any]:
    params = build_args(module)
    results = describe_ec2_dedicated_hosts(client, **params)
    return dict(hosts=format_results(results))


def list_mac_dedicated_hosts(client: Any, module: AnsibleAWSModule) -> Dict[str, Any]:
    params = build_args(module)
    results = describe_ec2_mac_dedicated_hosts(client, **params)
    return dict(mac_hosts=format_results(results))


def main():
    module = AnsibleAWSModule(
        argument_spec=dict(
            host_ids=dict(type="list", elements="str"),
            filters=dict(type="dict"),
            mac_only=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
    )

    client = module.client("ec2")
    mac_only = module.params.get("mac_only")

    if mac_only:
        result = list_mac_dedicated_hosts(client, module)
    else:
        result = list_dedicated_hosts(client, module)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2024 Ansible Project
# Copyright (c) 2024 Mandar Vijay Kulkarni (@mandar242)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: rds_instance_param_group_info
version_added: 9.1.0
short_description: Describes the RDS parameter group.
description:
  - Describe a specific RDS parameter group, or
    all parameter groups available in the current region.
options:
  db_parameter_group_name:
    description:
      - The name of a specific DB parameter group to return details for.
    required: false
    type: str
author:
  - Mandar Vijay Kulkarni (@mandar242)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get specific DB parameter group's info
  amazon.aws.rds_instance_param_group_info:
    db_parameter_group_name: my-test-pg

- name: Get all parameter group info from the region
  amazon.aws.rds_instance_param_group_info:
"""

RETURN = r"""
db_instance_parameter_groups:
  description: List of RDS instance parameter groups.
  returned: always
  type: list
  contains:
    db_parameter_group_name:
      description:
        - The name of the RDS instance parameter group.
      type: str
    db_parameter_group_family:
      description:
        - The name of the RDS parameter group family that this RDS instance parameter group is compatible with.
      type: str
    description:
      description:
        - Provides the customer-specified description for this RDS instance parameter group.
      type: str
    db_parameter_group_arn:
      description:
        - The Amazon Resource Name (ARN) for the RDS instance parameter group.
      type: str
    tags:
      description: Any tags associated to the parameter group.
      returned: always
      type: dict
      sample: {
                "Name": "test-parameter-group",
                "Env": "Dev001"
              }
"""

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_instance_parameter_groups


def main() -> None:
    argument_spec = {
        "db_parameter_group_name": {"type": "str", "required": False},
    }

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = module.client("rds")
    db_parameter_group_name = module.params.get("db_parameter_group_name")

    result = describe_db_instance_parameter_groups(client, module, db_parameter_group_name)

    module.exit_json(changed=False, db_instance_parameter_groups=result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_job_info
version_added: "3.17.0"
short_description: List ML Job resources
description:
    - List ML Job resources.
options:
    name:
        description:
            - Name of the Azure ML job.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    parent_job_name:
        description:
            - Name of the parent job. Will list all jobs whose
              parent_job_name matches the given name.
        required: false
        type: str
    list_type:
        description:
            - Specify if you want active, archived or all jobs.
        type: str
        choices:
            - active
            - archived
            - all

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: List ML Job by name
  azure.azcollection.azure_rm_ml_job_info:
    name: jobxxxxxxxxxx
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List Archived ML Jobs
  azure.azcollection.azure_rm_ml_job_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    list_type: archived
'''

RETURN = '''
ml_jobs:
    description:
        - Jobs that match the query.
    returned: always
    type: dict
    sample: [
      {
          "code": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-job/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/codes/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/versions/1",
          "command": "python train.py --data_dir ${{inputs.fashion_mnist}} --model_dir ${{outputs.model}}",
          "compute": "azureml:cluster-cpu",
          "creation_context": {
              "created_at": "2026-03-31T17:55:36.317328+00:00",
              "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "created_by_type": "Application"
          },
          "description": "Trains a simple neural network on the Fashion-MNIST dataset.",
          "display_name": "jobxxxxxxxxxx",
          "environment": "azureml:CliV2AnonymousEnvironment:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
          "environment_variables": {},
          "experiment_name": "aml_command_cli",
          "id": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-job/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/jobs/jobxxxxxxxxxx",
          "inputs": {
              "fashion_mnist": {
                  "mode": "ro_mount",
                  "path": "azureml:data-fashion-mnist:1",
                  "type": "uri_folder"
              }
          },
          "name": "jobxxxxxxxxxx",
          "outputs": {
              "default": {
                  "mode": "rw_mount",
                  "path": "azureml://datastores/workspaceartifactstore/ExperimentRun/dcid.jobxxxxxxxxxx",
                  "type": "uri_folder"
              },
              "model": {
                  "mode": "rw_mount",
                  "type": "mlflow_model"
              }
          },
          "parameters": {},
          "properties": {
              "ContentSnapshotId": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "EndTimeUtc": "2026-03-31 18:18:41",
              "ProcessInfoFile": "azureml-logs/process_info.json",
              "ProcessStatusFile": "azureml-logs/process_status.json",
              "StartTimeUtc": "2026-03-31 18:14:56",
              "_azureml.ClusterName": "cluster-cpu",
              "_azureml.ComputeTargetType": "amlctrain"
          },
          "queue_settings": {
              "job_tier": "null"
          },
          "resources": {
              "instance_count": 1,
              "properties": {},
              "shm_size": "2g"
          },
          "services": {
              "Studio": {
                  "endpoint": "https://ml.azure.com/runs/jobxxxxxxxxxx?wsid=/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/xxxxx-ml-job/workspaces/workspace-xxxxxxxxxx&tid=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                  "type": "Studio"
              },
              "Tracking": {
                  "endpoint": "azureml://eastus.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-job/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx?",
                  "type": "Tracking"
              }
          },
          "status": "Completed",
          "tags": {},
          "type": "command"
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLJobInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            parent_job_name=dict(
                type='str',
                required=False,
            ),
            list_type=dict(
                type='str',
                required=False,
                choices=['archived', 'active', 'all'],
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_jobs=[]
        )

        required_one_of = [('name', 'list_type')]
        mutually_exclusive = [('name', 'list_type'),
                              ('name', 'parent_job_name')]

        super(AzureRMMLJobInfo, self).__init__(self.module_arg_spec,
                                               supports_tags=False,
                                               supports_check_mode=True,
                                               facts_module=True,
                                               required_one_of=required_one_of,
                                               mutually_exclusive=mutually_exclusive,
                                               )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.jobs.get(name=self.name)
                ml_jobs = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_jobs = []
        else:
            list_view_type = self.get_list_view_type(self.list_type)
            results = self.client.jobs.list(parent_job_name=self.parent_job_name,
                                            list_view_type=list_view_type)
            ml_jobs = [self.entity_to_dict(self.client.jobs.get(x.name)) for x in results]

        self.results['ml_jobs'] = ml_jobs
        return self.results


def main():
    AzureRMMLJobInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# Copyright (c) 2021, Christophe Gilles <christophe.gilles54@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or
# https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_authentication_required_actions

short_description: Allows administration of Keycloak authentication required actions

description:
  - This module can register, update and delete required actions.
  - It also filters out any duplicate required actions by their alias. The first occurrence is preserved.
version_added: 7.1.0

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0

options:
  realm:
    description:
      - The name of the realm in which are the authentication required actions.
    required: true
    type: str
  required_actions:
    elements: dict
    description:
      - Authentication required action.
    suboptions:
      alias:
        description:
          - Unique name of the required action.
        required: true
        type: str
      config:
        description:
          - Configuration for the required action.
        type: dict
      defaultAction:
        description:
          - Indicates whether new users have the required action assigned to them.
        type: bool
      enabled:
        description:
          - Indicates, if the required action is enabled or not.
        type: bool
      name:
        description:
          - Displayed name of the required action. Required for registration.
        type: str
      priority:
        description:
          - Priority of the required action.
        type: int
      providerId:
        description:
          - Provider ID of the required action. Required for registration.
        type: str
    type: list
  state:
    choices: ["absent", "present"]
    description:
      - Control if the realm authentication required actions are going to be registered/updated (V(present)) or deleted (V(absent)).
    required: true
    type: str

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Skrekulko (@Skrekulko)
"""

EXAMPLES = r"""
- name: Register a new required action.
  community.general.keycloak_authentication_required_actions:
    auth_client_id: "admin-cli"
    auth_keycloak_url: "http://localhost:8080"
    auth_password: "password"
    auth_realm: "master"
    auth_username: "admin"
    realm: "master"
    required_actions:
      - alias: "TERMS_AND_CONDITIONS"
        name: "Terms and conditions"
        providerId: "TERMS_AND_CONDITIONS"
        enabled: true
    state: "present"

- name: Update the newly registered required action.
  community.general.keycloak_authentication_required_actions:
    auth_client_id: "admin-cli"
    auth_keycloak_url: "http://localhost:8080"
    auth_password: "password"
    auth_realm: "master"
    auth_username: "admin"
    realm: "master"
    required_actions:
      - alias: "TERMS_AND_CONDITIONS"
        enabled: false
    state: "present"

- name: Delete the updated registered required action.
  community.general.keycloak_authentication_required_actions:
    auth_client_id: "admin-cli"
    auth_keycloak_url: "http://localhost:8080"
    auth_password: "password"
    auth_realm: "master"
    auth_username: "admin"
    realm: "master"
    required_actions:
      - alias: "TERMS_AND_CONDITIONS"
    state: "absent"
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str

end_state:
  description: Representation of the authentication required actions after module execution.
  returned: on success
  type: dict
  contains:
    alias:
      description:
        - Unique name of the required action.
      sample: test-provider-id
      type: str
    config:
      description:
        - Configuration for the required action.
      sample: {}
      type: dict
    defaultAction:
      description:
        - Indicates whether new users have the required action assigned to them.
      sample: false
      type: bool
    enabled:
      description:
        - Indicates, if the required action is enabled or not.
      sample: false
      type: bool
    name:
      description:
        - Displayed name of the required action. Required for registration.
      sample: Test provider ID
      type: str
    priority:
      description:
        - Priority of the required action.
      sample: 90
      type: int
    providerId:
      description:
        - Provider ID of the required action. Required for registration.
      sample: test-provider-id
      type: str
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    keycloak_argument_spec,
)


def sanitize_required_actions(objects):
    for obj in objects:
        alias = obj["alias"]
        name = obj["name"]
        provider_id = obj["providerId"]

        if not name:
            obj["name"] = alias

        if provider_id != alias:
            obj["providerId"] = alias

    return objects


def filter_duplicates(objects):
    filtered_objects = {}

    for obj in objects:
        alias = obj["alias"]

        if alias not in filtered_objects:
            filtered_objects[alias] = obj

    return list(filtered_objects.values())


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        realm=dict(type="str", required=True),
        required_actions=dict(
            type="list",
            elements="dict",
            options=dict(
                alias=dict(type="str", required=True),
                config=dict(type="dict"),
                defaultAction=dict(type="bool"),
                enabled=dict(type="bool"),
                name=dict(type="str"),
                priority=dict(type="int"),
                providerId=dict(type="str"),
            ),
        ),
        state=dict(type="str", choices=["present", "absent"], required=True),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", end_state={}, diff=dict(before={}, after={}))

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    # Convenience variables
    realm = module.params.get("realm")
    desired_required_actions = module.params.get("required_actions")
    state = module.params.get("state")

    # Sanitize required actions
    desired_required_actions = sanitize_required_actions(desired_required_actions)

    # Filter out duplicate required actions
    desired_required_actions = filter_duplicates(desired_required_actions)

    # Get required actions
    before_required_actions = kc.get_required_actions(realm=realm)

    if state == "present":
        # Initialize empty lists to hold the required actions that need to be
        # registered, updated, and original ones of the updated one
        register_required_actions = []
        before_updated_required_actions = []
        updated_required_actions = []

        # Loop through the desired required actions and check if they exist in the before required actions
        for desired_required_action in desired_required_actions:
            found = False

            # Loop through the before required actions and check if the aliases match
            for before_required_action in before_required_actions:
                if desired_required_action["alias"] == before_required_action["alias"]:
                    update_required = False

                    # Fill in the parameters
                    for k, v in before_required_action.items():
                        if k not in desired_required_action or desired_required_action[k] is None:
                            desired_required_action[k] = v

                    # Loop through the keys of the desired and before required actions
                    # and check if there are any differences between them
                    for key in desired_required_action.keys():
                        if (
                            key in before_required_action
                            and desired_required_action[key] != before_required_action[key]
                        ):
                            update_required = True
                            break

                    # If there are differences, add the before and desired required actions
                    # to their respective lists for updating
                    if update_required:
                        before_updated_required_actions.append(before_required_action)
                        updated_required_actions.append(desired_required_action)
                    found = True
                    break
            # If the desired required action is not found in the before required actions,
            # add it to the list of required actions to register
            if not found:
                # Check if name is provided
                if "name" not in desired_required_action or desired_required_action["name"] is None:
                    module.fail_json(
                        msg=f"Unable to register required action {desired_required_action['alias']} in realm {realm}: name not included"
                    )

                # Check if provider ID is provided
                if "providerId" not in desired_required_action or desired_required_action["providerId"] is None:
                    module.fail_json(
                        msg=f"Unable to register required action {desired_required_action['alias']} in realm {realm}: providerId not included"
                    )

                register_required_actions.append(desired_required_action)

        # Handle diff
        if module._diff:
            diff_required_actions = updated_required_actions.copy()
            diff_required_actions.extend(register_required_actions)

            result["diff"] = dict(before=before_updated_required_actions, after=diff_required_actions)

        # Handle changed
        if register_required_actions or updated_required_actions:
            result["changed"] = True

        # Handle check mode
        if module.check_mode:
            if register_required_actions or updated_required_actions:
                result["change"] = True
                result["msg"] = "Required actions would be registered/updated"
            else:
                result["change"] = False
                result["msg"] = "Required actions would not be registered/updated"

            module.exit_json(**result)

        # Register required actions
        if register_required_actions:
            for register_required_action in register_required_actions:
                kc.register_required_action(realm=realm, rep=register_required_action)
                kc.update_required_action(
                    alias=register_required_action["alias"], realm=realm, rep=register_required_action
                )

        # Update required actions
        if updated_required_actions:
            for updated_required_action in updated_required_actions:
                kc.update_required_action(
                    alias=updated_required_action["alias"], realm=realm, rep=updated_required_action
                )

        # Initialize the final list of required actions
        final_required_actions = []

        # Iterate over the before_required_actions
        for before_required_action in before_required_actions:
            # Check if there is an updated_required_action with the same alias
            updated_required_action_found = False

            for updated_required_action in updated_required_actions:
                if updated_required_action["alias"] == before_required_action["alias"]:
                    # Merge the two dictionaries, favoring the values from updated_required_action
                    merged_dict = {}
                    for key in before_required_action.keys():
                        if key in updated_required_action:
                            merged_dict[key] = updated_required_action[key]
                        else:
                            merged_dict[key] = before_required_action[key]

                    for key in updated_required_action.keys():
                        if key not in before_required_action:
                            merged_dict[key] = updated_required_action[key]

                    # Add the merged dictionary to the final list of required actions
                    final_required_actions.append(merged_dict)

                    # Mark the updated_required_action as found
                    updated_required_action_found = True

                    # Stop looking for updated_required_action
                    break

            # If no matching updated_required_action was found, add the before_required_action to the final list of required actions
            if not updated_required_action_found:
                final_required_actions.append(before_required_action)

        # Append any remaining updated_required_actions that were not merged
        for updated_required_action in updated_required_actions:
            if not any(updated_required_action["alias"] == action["alias"] for action in final_required_actions):
                final_required_actions.append(updated_required_action)

        # Append newly registered required actions
        final_required_actions.extend(register_required_actions)

        # Handle message and end state
        result["msg"] = "Required actions registered/updated"
        result["end_state"] = final_required_actions
    else:
        # Filter out the deleted required actions
        final_required_actions = []
        delete_required_actions = []

        for before_required_action in before_required_actions:
            delete_action = False

            for desired_required_action in desired_required_actions:
                if before_required_action["alias"] == desired_required_action["alias"]:
                    delete_action = True
                    break

            if not delete_action:
                final_required_actions.append(before_required_action)
            else:
                delete_required_actions.append(before_required_action)

        # Handle diff
        if module._diff:
            result["diff"] = dict(before=before_required_actions, after=final_required_actions)

        # Handle changed
        if delete_required_actions:
            result["changed"] = True

        # Handle check mode
        if module.check_mode:
            if final_required_actions:
                result["change"] = True
                result["msg"] = "Required actions would be deleted"
            else:
                result["change"] = False
                result["msg"] = "Required actions would not be deleted"

            module.exit_json(**result)

        # Delete required actions
        if delete_required_actions:
            for delete_required_action in delete_required_actions:
                kc.delete_required_action(alias=delete_required_action["alias"], realm=realm)

        # Handle message and end state
        result["msg"] = "Required actions deleted"
        result["end_state"] = final_required_actions

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2026, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.general.plugins.module_utils import _deps as deps

nomad: t.Any = None
with deps.declare("python-nomad"):
    import nomad  # type: ignore[no-redef]

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

argument_spec = dict(
    host=dict(required=True, type="str"),
    port=dict(type="int", default=4646),
    use_ssl=dict(type="bool", default=True),
    timeout=dict(type="int", default=5),
    validate_certs=dict(type="bool", default=True),
    client_cert=dict(type="path"),
    client_key=dict(type="path"),
    namespace=dict(type="str"),
    token=dict(type="str", no_log=True),
)


def setup_nomad_client(module: AnsibleModule) -> nomad.Nomad:
    deps.validate(module)

    certificate_ssl = (module.params["client_cert"], module.params["client_key"])

    return nomad.Nomad(
        host=module.params["host"],
        port=module.params["port"],
        secure=module.params["use_ssl"],
        timeout=module.params["timeout"],
        verify=module.params["validate_certs"],
        cert=certificate_ssl,
        namespace=module.params["namespace"],
        token=module.params["token"],
    )



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._names import (
    join_labels,
    normalize_label,
    split_into_labels,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    DNSAPIError,
)

if t.TYPE_CHECKING:
    from .._provider import ProviderInformation  # pragma: no cover


@t.overload
def normalize_dns_name(name: str) -> str: ...


@t.overload
def normalize_dns_name(name: str | None) -> str | None: ...


def normalize_dns_name(name: str | None) -> str | None:
    if name is None:
        return name
    labels, dummy = split_into_labels(name)
    return join_labels([normalize_label(label) for label in labels])


def get_prefix(
    normalized_zone: str,
    provider_information: ProviderInformation,
    *,
    normalized_record: str | None = None,
    prefix: str | None = None
) -> tuple[str, str | None]:
    # If normalized_record is not specified, use prefix
    if normalized_record is None:
        if prefix is not None:
            prefix = provider_information.normalize_prefix(normalize_dns_name(prefix))
        return (prefix + "." + normalized_zone) if prefix else normalized_zone, prefix
    # Convert record to prefix
    if (
        not normalized_record.endswith("." + normalized_zone)
        and normalized_record != normalized_zone
    ):
        raise DNSAPIError("Record must be in zone")
    if normalized_record == normalized_zone:
        return normalized_record, None
    return (
        normalized_record,
        normalized_record[: len(normalized_record) - len(normalized_zone) - 1],
    )
